#!/usr/bin/env python3

import argparse
import base64
import hashlib
import json
import os
import requests
from subprocess import Popen, PIPE, STDOUT
from dotenv import load_dotenv
load_dotenv()

SOURCEDIR = os.environ['SOURCEDIR']
SOURCEURL = os.environ['SOURCEURL']
DRUPALURL = os.environ['DRUPALURL']
DRUPALUSER = os.environ['DRUPALUSER']
DRUPALPASS = os.environ['DRUPALPASS']
DRUPALUUID = os.environ['DRUPALUUID']
NEXTCLOUD_APP_BASE = os.environ['NEXTCLOUD_APP_PATH']
NEXTCLOUD_DATA_BASE = os.environ['NEXTCLOUD_DATA_PATH']
NEXTCLOUD_GROUPFOLDERS = '__groupfolders'

parser = argparse.ArgumentParser(description='Nextcloud to Drupal')
parser.add_argument('id', help='Nextcloud ID')
parser.add_argument('path', help='File path')
args, extras = parser.parse_known_args()

appPath = '/'.join([NEXTCLOUD_APP_BASE, args.id])
groups = []
process = Popen(['./occ', 'groupfolders:list', '--output=json'], stdout=PIPE, stderr=STDOUT, cwd=appPath)
process.wait()
for line in process.stdout.read().decode('utf8').split("\n"):
    if line != '' and line[0] == '[':
        groups = json.loads(line)

parts = args.path.split("/")
parts[0] = '/'.join([NEXTCLOUD_DATA_BASE, args.id])
print('Org path: {}'.format('/'.join(parts)))

for group in groups:
    print(group)
    if group['mount_point'] == parts[3]:
        parts[2] = NEXTCLOUD_GROUPFOLDERS
        parts[3] = str(group['id'])
        del(parts[1])

realPath = '/'.join(parts)
realPathHash = hashlib.md5(realPath.encode()).hexdigest()
linkPath = ''.join([SOURCEDIR, realPathHash, parts[-1]])
print('Real path: {}'.format(realPath))

process = Popen(['ln', '-s', realPath, linkPath], stdout=PIPE, stderr=STDOUT)
process.wait()

auth = ':'.join([DRUPALUSER, DRUPALPASS])
encodedAuthBytes = base64.b64encode(auth.encode("utf-8"))
encodedAuthStr = str(encodedAuthBytes, "utf-8")

header = {
    "Accept": "application/vnd.api+json",
    "Content-Type": "application/vnd.api+json",
    "Authorization": "Basic {}".format(encodedAuthStr)
}
payload = {
    "data": {
        "type": "file--file",
        "attributes": {
            "filename": parts[-1],
            "uri": ''.join([SOURCEURL, '/', realPathHash, parts[-1]])
        },
        "relationships": {
            "uid": {
                "data": {
                    "type": "user--user",
                    "id": DRUPALUUID
                }
            }
        }
    }
}
r = requests.post('{}/jsonapi/file/file'.format(DRUPALURL), data=json.dumps(payload), headers=header)
file = json.loads(r.text)
print(file)

if 'data' in file:
    payload = {
        "data": {
            "type": "media--remote_image",
            "attributes": {
                "name": parts[-1]
            },
            "relationships": {
                "field_media_image": {
                    "data": {
                        "type": "file--file",
                        "id": file['data']['id']
                    }
                }
            }
        }
    }
    r = requests.post('{}/jsonapi/media/remote_image'.format(DRUPALURL), data=json.dumps(payload), headers=header)
    media = json.loads(r.text)
    print(media)
